"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateAction = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const abstract_action_1 = require("../lib/common/abstract.action");
const executor_1 = require("../lib/executor");
const db_datasource_1 = require("../lib/utils/db.datasource");
const is_1 = require("../lib/utils/is");
const fs_extra_1 = require("fs-extra");
const inquirer = require("inquirer");
const doc_type_1 = require("../lib/constants/doc-type");
const configuration_load_1 = require("../lib/configuration/configuration.load");
const writer_1 = require("../lib/writer");
class GenerateAction extends abstract_action_1.AbstractAction {
    handle(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let docType = inputs.find((e) => e.name === 'type').value;
                const output = options.find((e) => e.name === 'output').value;
                const force = options.find((e) => e.name === 'force').value;
                // 加载MYSQL配置
                const sqlconf = (0, configuration_load_1.loadDatabaseConfiguration)(inputs.concat(options));
                // 生成的文档类型, 如果未填写则让尝试选择生成的类型
                if (!docType) {
                    const { type } = yield inquirer.prompt([
                        {
                            type: 'list',
                            message: 'Select the type of document you want to generate',
                            name: 'type',
                            choices: doc_type_1.DocTypeList,
                        },
                    ]);
                    docType = type;
                }
                // 初始化mysql数据库连接
                yield db_datasource_1.DBDataSource.getInstance().init(sqlconf);
                // 测试连接查询mysql版本
                const [{ version }] = yield new executor_1.VersionExecutor().exec();
                console.log(`\nMysql connection is successful. database version is ${version}`);
                // 数据库名称
                const database = sqlconf.database;
                // 处理输出文件的路径
                let path = process.cwd();
                if ((0, is_1.isString)(output) && output) {
                    // 处理相对路径
                    path = output.startsWith('.') ? (0, path_1.resolve)(path, output) : output;
                    // 强制创建路径
                    (0, fs_extra_1.ensureDirSync)(path);
                }
                // 文件路径
                const filePath = (0, path_1.join)(path, `${database}.${docType}`);
                // 当文件已存在，提示是否需要覆盖
                if ((0, fs_extra_1.existsSync)(filePath) && !force) {
                    const { ok } = yield inquirer.prompt([
                        {
                            type: 'confirm',
                            message: 'Whether to overwrite the existing file',
                            name: 'ok',
                        },
                    ]);
                    if (!ok) {
                        console.log(`\nNothing to do!`);
                        process.exit(1);
                    }
                }
                console.log('\nLooking for all tables in the database...');
                const tables = yield new executor_1.TablesExecutor(database).exec();
                if (tables.length <= 0) {
                    console.log('\nTable is empty, no documents need to be generated!');
                    process.exit(1);
                }
                (0, is_1.isDebug)() && console.table(tables);
                // 创建流
                const sb = (0, fs_1.createWriteStream)(filePath, {
                    start: 0,
                    encoding: 'utf8',
                });
                // 处理文档内容
                let writer = null;
                if (docType === 'docx') {
                    writer = new writer_1.DocxWriter(sb, database, tables);
                }
                else {
                    writer = new writer_1.MarkdownWriter(sb, database, tables);
                }
                yield writer.write();
                // 结束流
                sb.end();
                console.log('\n√ Document has been generated in ' + filePath);
            }
            finally {
                // destroy connection
                db_datasource_1.DBDataSource.getInstance().destroy();
            }
        });
    }
}
exports.GenerateAction = GenerateAction;
