"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocxWriter = void 0;
const abstract_writer_1 = require("../common/abstract.writer");
const docx_1 = require("docx");
const executor_1 = require("../executor");
const table_1 = require("../constants/table");
const formatter_1 = require("../utils/formatter");
class DocxWriter extends abstract_writer_1.AbstractWriter {
    write() {
        return __awaiter(this, void 0, void 0, function* () {
            const tableComp = [];
            for (let i = 0; i < this.tables.length; i++) {
                tableComp.push(new docx_1.Paragraph({
                    heading: docx_1.HeadingLevel.HEADING_2,
                    children: [
                        new docx_1.TextRun({
                            text: this.tables[i].name,
                            bold: true,
                        }),
                    ],
                }));
                tableComp.push(new docx_1.Paragraph({}));
                const comment = this.tables[i].comment;
                if (comment) {
                    tableComp.push(new docx_1.Paragraph({
                        children: [
                            new docx_1.TextRun({
                                text: `表注释: ${comment}`,
                                color: '#24292f',
                            }),
                        ],
                    }));
                    tableComp.push(new docx_1.Paragraph({}));
                }
                const t = yield this.createTableDetail(this.tables[i]);
                tableComp.push(t);
                tableComp.push(new docx_1.Paragraph({}));
                tableComp.push(new docx_1.Paragraph({
                    children: [
                        new docx_1.TextRun({
                            text: '索引',
                            bold: true,
                        }),
                    ],
                }));
                tableComp.push(new docx_1.Paragraph({}));
                const t2 = yield this.createIndexdDetail(this.tables[i]);
                tableComp.push(t2);
                tableComp.push(new docx_1.Paragraph({}));
            }
            const doc = new docx_1.Document({
                sections: [
                    {
                        children: [
                            new docx_1.Paragraph({
                                heading: docx_1.HeadingLevel.HEADING_1,
                                alignment: docx_1.AlignmentType.CENTER,
                                children: [
                                    new docx_1.TextRun({
                                        bold: true,
                                        text: `${this.database}数据库文档`,
                                    }),
                                ],
                            }),
                            new docx_1.Paragraph({}),
                            ...tableComp,
                        ],
                    },
                ],
            });
            // 写入文件
            const buffer = yield docx_1.Packer.toBuffer(doc);
            this.stream.write(buffer);
        });
    }
    createIndexdDetail(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const indexd = yield new executor_1.IndexdExecutor(this.database, table.name).exec();
            const rows = indexd.map((c) => {
                return new docx_1.TableRow({
                    children: table_1.indexdKeys.map((k) => {
                        return new docx_1.TableCell({
                            verticalAlign: docx_1.VerticalAlign.CENTER,
                            children: [
                                new docx_1.Paragraph({
                                    children: [
                                        new docx_1.TextRun({
                                            text: (0, formatter_1.formatterIndexColValue)(c, k),
                                        }),
                                    ],
                                    alignment: docx_1.AlignmentType.CENTER,
                                }),
                            ],
                        });
                    }),
                });
            });
            return new docx_1.Table({
                rows: [
                    new docx_1.TableRow({
                        children: table_1.indexdTitles.map((e) => {
                            return new docx_1.TableCell({
                                verticalAlign: docx_1.VerticalAlign.CENTER,
                                children: [
                                    new docx_1.Paragraph({
                                        children: [
                                            new docx_1.TextRun({
                                                text: e,
                                                bold: true,
                                            }),
                                        ],
                                        alignment: docx_1.AlignmentType.CENTER,
                                    }),
                                ],
                            });
                        }),
                    }),
                    ...rows,
                ],
                width: {
                    size: 100,
                    type: docx_1.WidthType.PERCENTAGE,
                },
            });
        });
    }
    createTableDetail(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const columns = yield new executor_1.ColumnsExecutor(this.database, table.name).exec();
            const rows = columns.map((col) => {
                return new docx_1.TableRow({
                    children: table_1.colKeys.map((key) => {
                        return new docx_1.TableCell({
                            verticalAlign: docx_1.VerticalAlign.CENTER,
                            children: [
                                new docx_1.Paragraph({
                                    children: [
                                        new docx_1.TextRun({
                                            text: col[key],
                                        }),
                                    ],
                                    alignment: docx_1.AlignmentType.CENTER,
                                }),
                            ],
                        });
                    }),
                });
            });
            return new docx_1.Table({
                rows: [
                    new docx_1.TableRow({
                        children: table_1.colTitles.map((e) => {
                            return new docx_1.TableCell({
                                verticalAlign: docx_1.VerticalAlign.CENTER,
                                children: [
                                    new docx_1.Paragraph({
                                        children: [
                                            new docx_1.TextRun({
                                                text: e,
                                                bold: true,
                                            }),
                                        ],
                                        alignment: docx_1.AlignmentType.CENTER,
                                    }),
                                ],
                            });
                        }),
                    }),
                    ...rows,
                ],
                width: {
                    size: 100,
                    type: docx_1.WidthType.PERCENTAGE,
                },
            });
        });
    }
}
exports.DocxWriter = DocxWriter;
