"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownWriter = void 0;
const abstract_writer_1 = require("../common/abstract.writer");
const executor_1 = require("../executor");
const table_1 = require("../constants/table");
const formatter_1 = require("../utils/formatter");
class MarkdownWriter extends abstract_writer_1.AbstractWriter {
    write() {
        return __awaiter(this, void 0, void 0, function* () {
            // title
            const docTitle = this.createTitle(`${this.database}数据库文档`, 1, false, '<a name="返回顶部"></a>');
            this.stream.write(docTitle);
            // outline
            const docOutline = this.createOutline(this.tables);
            this.stream.write(docOutline);
            // tables
            for (let i = 0; i < this.tables.length; i++) {
                const tableTitle = this.createTitle([this.tables[i].name, '', `<a name="${this.tables[i].name}"></a>`].join('\n'), 2, true, this.tables[i].comment
                    ? `> 表注释: ${this.tables[i].comment}`
                    : undefined);
                this.stream.write(tableTitle);
                const tableDesc = yield this.createTable(this.tables[i]);
                this.stream.write(tableDesc);
            }
        });
    }
    createTitle(title, level, newLine = false, subTitle) {
        const text = [];
        if (newLine) {
            text.push('');
        }
        text.push(...[`${'#'.repeat(level)} ${title}`, '']);
        if (subTitle) {
            text.push(...[subTitle, '']);
        }
        text.push('');
        return text.join('\n');
    }
    createOutline(tables) {
        const outline = [];
        tables.forEach((e) => {
            const title = `* [${e.name}](#${e.name})`;
            outline.push(...[title, '']);
        });
        return [...outline].join('\n');
    }
    createTable(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const text = [];
            // columns
            const columns = yield new executor_1.ColumnsExecutor(this.database, table.name).exec();
            text.push(['', ...table_1.colTitles, ''].join('|'));
            text.push(['', ...table_1.colTitles.map(() => ':---:'), ''].join('|'));
            columns.forEach((c) => {
                const values = table_1.colKeys.map((k) => c[k]);
                text.push(['', ...values, ''].join('|'));
            });
            // indexd
            const indexd = yield new executor_1.IndexdExecutor(this.database, table.name).exec();
            if (indexd.length > 0) {
                // new line
                text.push(...['', '**索引**', '']);
                text.push(['', ...table_1.indexdTitles, ''].join('|'));
                text.push(['', ...table_1.indexdTitles.map(() => ':---:'), ''].join('|'));
                indexd.forEach((c) => {
                    const values = table_1.indexdKeys.map((k) => (0, formatter_1.formatterIndexColValue)(c, k));
                    text.push(['', ...values, ''].join('|'));
                });
            }
            // new line
            text.push('');
            return text.join('\n');
        });
    }
}
exports.MarkdownWriter = MarkdownWriter;
